<?php

  class Base {

    protected $_table_name;
    protected $_class_name;

    protected $user_id;
    protected $create_date;
    protected $update_date;
    
    public function to_object($array){
      $fields = array_keys(get_object_vars($this));
      foreach ( $array as $key => $data ){
        if ( in_array($key, $fields) ){          
          $this->$key = $data;
        }
        
      }

      return $this;
    }

    public function to_array(){            
      return get_object_vars($this);    
    }
    
    public static function get_class_vars($object){
      return array_keys(get_class_vars(get_class($object))); // $object
    }

    public static function reorder_sql_filters($filters = []){ // ordering and limit should be last keys 
      $last_key_order = [ "order", "limit" ];
      foreach ( $last_key_order as $key ){
        if ( isset($filters[$key]) ){
          $temp = $filters[$key];
          unset($filters[$key]);
          $filters[$key] = $temp;
        }
      }

      return $filters;
    }

    public function create(){            
      $Crud = new Crud();
      $new_data = $this->to_array();
      unset($new_data['create_date']);
      unset($new_data['id']);
      return $Crud->create($table = $this->get__table_name(), $new_data );
    }

    public function edit(){
      $Crud = new Crud(); 
      $new_data = $this->to_array();
      $id = $this->get_id();
      unset($new_data['id']);

      $data = $this->list(["id" => $id, "limit" => 1]);
      
      if ( !$data )
        throw new NotFoundException("{$this->get__class_name()} id: {$id} não encontrado, impossível editar o registro");

      return $Crud->update($table = $this->get__table_name(), $new_data, $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    public function delete($id){
      $data = $this->list(["id" => $id, "limit" => 1]);
    
      if ( !$data )
        throw new NotFoundException("{$this->get__class_name()} id: {$id} não encontrado, impossível excluir o registro");

      $Crud = new Crud();
      return $Crud->delete($table = $this->get__table_name(), $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    protected function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $filters = self::reorder_sql_filters($filters);
      $class_vars = self::get_class_vars($this);

      if ( $class_vars ){
        $Crud = new Crud();
        $class_vars = $Crud->get_fields(['table' => $this->get__table_name(), 'ignore_fields' => null ]);
      }
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND " . $this->get__table_name() .".{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "order":
                $where .= "ORDER BY {$filter}";
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }

    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $field_list = "*";
      if ( !isset($filters['return_all_fields']) || $filters['return_all_fields'] != true ){ 
        if ( !isset($filters['ignore_fields']) || empty($filters['ignore_fields']) ){ // remove
          $ignore_fields = [ ];
        }else {
          $ignore_fields = $filters['ignore_fields'];
        }
        $field_list = $Crud->get_fields(["table" => $this->get__table_name(), "ignore_fields" => $ignore_fields, "return_query_field_list" => true ] );
      }

      $bind = [];
      if ( $filters ){
        $sql_filter = $this->create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = $this->get__table_name(), $where, $bind, $fields = $field_list);
    }

    // default attributes
    public function get_user_id(){ return $this->user_id; }
    public function set_user_id($user_id): self { $this->user_id = $user_id; return $this; }

    public function get_create_date(){ return $this->create_date; }
    public function set_create_date($create_date): self { $this->create_date = $create_date; return $this; }

    public function get_update_date(){ return $this->update_date; }
    public function set_update_date($update_date): self { $this->update_date = $update_date; return $this; }

    public function get__table_name(){ return $this->_table_name; }
    public function set__table_name($_table_name): self { $this->_table_name = $_table_name; return $this; }
    
    public function get__class_name(){ return $this->_class_name; }
    public function set__class_name($_class_name): self { $this->_class_name = $_class_name; return $this; }

  }