<?php

  Class Body {

    private $id;
    private $create_time;
    private $data;
    private $tracking_code_id;
  
    private $table_name = "bodies";

    public function get_id(){ return $this->id; }
    public function set_id($id): self { $this->id = $id; return $this; }

    public function get_create_time(){ return $this->create_time; }
    public function set_create_time($create_time): self { $this->create_time = $create_time; return $this; }

    public function get_data(){ return $this->data; }
    public function set_data($data): self { $this->data = $data; return $this; }

    public function get_tracking_code_id(){ return $this->tracking_code_id; }
    public function set_tracking_code_id($tracking_code_id): self { $this->tracking_code_id = $tracking_code_id; return $this; }

    public function get_table_name(){ return $this->table_name; }

    public function to_object($array){
      $fields = array_keys(get_object_vars($this));
      foreach ( $array as $key => $data ){
        if ( in_array($key, $fields) ){          
          $this->$key = $data;
        }
        
      }

      return $this;
    }

    public function to_array(){            
      return get_object_vars($this);    
    }

    public static function get_class_vars($object){
      return array_keys(get_class_vars(get_class($object))); // $object
    }

    public function create(){            
      $Crud = new Crud();
      $new_data = ["data" => json_encode($this->get_data(), true)];
      unset($new_data['create_date']);
      unset($new_data['id']);
      return $Crud->create($table = $this->get_table_name(), $new_data );
    }

    public function edit(){
      $Crud = new Crud(); 
      $new_data = ["data" => json_encode($this->get_data(), true)];
      $id = $this->get_id();
      unset($new_data['id']);

      return $Crud->update($table = $this->get_table_name(), $new_data, $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    public function delete($id){
      $data = $this->list(["id" => $id, "limit" => 1]);
      
      if ( !$data )
        throw new NotFoundException("Processo id: {$id} não encontrado, impossível excluir o registro");

      $Crud = new Crud();
      return $Crud->delete($table = $this->get_table_name(), $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    private function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $class_vars = self::get_class_vars($this);
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND {$this->get_table_name()}.{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "order":
                $where .= "ORDER BY :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }

    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $field_list = "*";
      if ( !isset($filters['return_all_fields']) || $filters['return_all_fields'] != true ){ 
        if ( !isset($filters['ignore_fields']) || empty($filters['ignore_fields']) ){ // remove password and token
          $ignore_fields = ["password", "api_token"];
        }else {
          $ignore_fields = $filters['ignore_fields'];
        }
        $field_list = $Crud->get_fields(["table" => $this->get_table_name(), "ignore_fields" => $ignore_fields, "return_query_field_list" => true ] );
      }

      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = $this->get_table_name(), $where, $bind, $fields = $field_list);
    }

    public static function get_form($mode = "create"){
      $form = [
          "id" => "form_body",
          "fields" => [
            ['id' => 'id', 'type' => 'hidden', 'required' => true],
            ['id' => 'record',  'label' => 'Atestado de Óbito', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 50, "placeholder" => "Informe o atestado de óbito"]],
            
            ['id' => 'company','label' => 'Empreendimento', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...'], 
              'options' => Company::get_list_options()], 
            ['id' => 'control', 'label' => 'Número de controle', 'required' => false,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 50] ],
            ['id' => 'name',  'label' => 'Nome', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 255] ],
            ['id' => 'ident',    'label' => 'Documento de Identidade', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 50] ],
            ['id' => 'gender','label' => 'Sexo', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...'], 'options' => ['male'=>'Masculino','female'=>'Feminino', 'none' => 'Indefinido']],
            ['id' => 'age',   'label' => 'Idade', 'required' => true,
              'type' => 'number', 'attributes' => ['min' => 0, 'max' => 200, 'suffix' => 'anos']],
            ['id' => 'height','label' => 'Altura', 'required' => true,
              'type' => 'number', 'attributes' => ['min' => 0, 'max' => 4, 'step' => 0.01, 'suffix' => 'm']],
            ['id' => 'weight','label' => 'Peso', 'required' => true,
              'type' => 'number', 'attributes' => ['min' => 0, 'max' => 1000, 'step' => 0.1, 'suffix' => 'kg']],
            ['id' => 'container','label' => 'Urna (Recipiente)', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...'], 'options' => ['varnish_laquer_coffin'=>'Madeira envernizada / laqueada','wood_coffin'=>'Madeira sem verniz','plywood'=>'Compensado de madeira','vinyl_bag'=>'Bolsa de vinil','wood_tray'=>'Bandeja de Madeira','cardboard'=>'Papelão']],
            ['id' => 'load_type','label' => 'Condição', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...'], 'options' => ['exhumation'=>'Exumação', 'cremation'=>'Cremação']],
            ['id' => 'tanatopraxy','label' => 'Tratamento', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...'], 'options' => ['none'=>'sem tanatopraxia', 'formol75dillution'=>'formol diluição 75%', 'formol50dillution'=>'formol diluição 50%', 'formol25dillution'=>'formol diluição 25%', 'formol'=>'formol concentrado']],
            ['id' => 'ashes','label' => 'Retorno das cinzas', 'required' => true,
              'type' => 'select', 'default' => 'yes', 'options' => ['no'=>'Não', 'yes'=>'Sim']],
            ['id' => 'individual','label' => 'Individual', 'required' => true,
              'type' => 'select', 'default' => 'yes', 'options' => ['no'=>'Não', 'yes'=>'Sim']],
            ['id' => 'comment',  'label' => 'Observações', 'required' => false,
             'type' => 'textarea'],

            
          ]
      ];

      switch ( $mode ){
        case "edit":
          $form['fields'][] = [
            "id" => "id", "name" => "id", "type" => "text", "label" => "Id",
            "required" => true ,
            "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
            "attributes" => ["min" => 1 ]
          ];
          break;
      }

      return $form;
    }

    public static function get_form_tracking_code($mode = "create"){
      $form = [
          "id" => "form_body_tracking_code",
          "fields" => [
            ['id' => 'id', 'type' => 'hidden', 'required' => true],
            ['id' => 'record',  'label' => 'Atestado de Óbito', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 50, 
              "readonly" => true,
              "placeholder" => "Informe o atestado de óbito"]],
            
            ['id' => 'company','label' => 'Empreendimento', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...',
              "readonly" => true, "disabled" => true], 
              'options' => Company::get_list_options()], 
            ['id' => 'control', 'label' => 'Número de controle', 'required' => false,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 50,
              "readonly" => true,] ],
            ['id' => 'name',  'label' => 'Nome', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 255,
              "readonly" => true,] ],
            ['id' => 'tracking_code',  'label' => 'Código de Rastreabilidade', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 10, "placeholder" => "Informe o código de rastreabilidade"]
            ],
            
          ]
      ];

      switch ( $mode ){
        case "edit":
          $form['fields'][] = [
            "id" => "id", "name" => "id", "type" => "text", "label" => "Id",
            "required" => true ,
            "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
            "attributes" => ["min" => 1 ]
          ];
          break;
      }

      return $form;
    }

    public function create_button($parameters = []){
      
      $buttons = [];
      foreach ( $parameters['action'] as $key => $action ){
        switch ( $action ){
          case  "edit":
            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_edit", "text-dark"), 
              "description" => Helper::get_icon(["icon" => "edit" ]) . " Editar ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                //"data-modal_description" => "Editar Processo: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                "data-bs-toggle" => "modal",
                "data-bs-target" => "#modal_form_process",    
                "data-form_id" => "process",           
                "data-action" => "edit",                       
              ],
              "href" => "#",
              "required_permission" => "body_edit",
            ];
            break;

          case  "delete":
            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_delete", "text-danger"), 
              "description" => Helper::get_icon(["icon" => "delete" ]) . " Excluir ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                "data-action" => "delete",
               // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                //"data-bs-toggle" => "modal",
                //"data-bs-target" => "#modal_form_process",                                      
              ],
              "href" => "#",
              "required_permission" => "body_delete",
            ];
            break;

            case  "add_tracking":
              $buttons[] = [ 
                "class" => array("api", "dropdown-item", "btn_add_tracking_code"), 
                "description" => Helper::get_icon(["icon" => "shield" ]) . " Vincular código de rastreabilidade ", 
                "type" => "link",
                "attributes" => [
                  "data-object" => get_class($this),
                  "data-action" => $action,
                  "data-id" => $parameters['data']['id'],
                  "data-object_info" => json_encode($parameters['data'], true),
                 // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                  "data-bs-toggle" => "modal",
                  "data-bs-target" => "#modal_form_process_tracking_code",         
                  "data-form_id" => "process_tracking_code",     
                  "data-action" => "add_tracking",                             
                ],
                "href" => "#",
                "required_permission" => "body_add_tracking",
              ];
              break;

            case  "print":
              $buttons[] = [ 
                "class" => array("api", "dropdown-item", "btn_print_cremation_certificate",), 
                "description" => Helper::get_icon(["icon" => "print" ]) . " Imprimir Autorização de Cremação ", 
                "type" => "link",
                "attributes" => [
                  "data-object" => get_class($this),
                  "data-action" => $action,
                  "data-id" => $parameters['data']['id'],
                  "data-object_info" => json_encode($parameters['data'], true),
                  // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                  //"data-bs-toggle" => "modal",
                  //"data-bs-target" => "#modal_form_process_tracking_code",                                      
                ],
                "href" => "#",
                "required_permission" => "body_add_tracking",
              ];
              break;

            case  "print_auth":
              $buttons[] = [ 
                "class" => array("api", "dropdown-item", "btn_print_cremation_authorization",), 
                "description" => Helper::get_icon(["icon" => "print" ]) . " Imprimir Certificado de Cremação ", 
                "type" => "link",
                "attributes" => [
                  "data-object" => get_class($this),
                  "data-action" => $action,
                  "data-id" => $parameters['data']['id'],
                  "data-object_info" => json_encode($parameters['data'], true),
                  // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                  //"data-bs-toggle" => "modal",
                  //"data-bs-target" => "#modal_form_process_tracking_code",                                      
                ],
                "href" => "#",
                "required_permission" => "body_add_tracking",
              ];
              break;
        }
      }

      if ( !$buttons )
        return "";
      
      return Helper::create_html_button_dropdown($buttons);

    }

    public function get_datatable_list($list){
      $return = [];
      $remove_columns = [
        //"batch", "start", "end", "create_time", "update_time", "data", "slots" 
      ];
      $show_columns = [ "tracking_code_id", "id", "name", "weight", "ashes", "individual" ];
      foreach ( $list as $key => $data ){
        $return[$key]['pending'] = "";
        $body_data = json_decode($data['data'], true);
        if ( !$body_data )
          continue;

        $return[$key] = array_merge(["id" => $data['id'] ], $body_data);

        foreach ( $body_data as $key2 => $column ){
          if ( !in_array($key2, $show_columns) )
            unset($return[$key][$key2]);
        }

        $return[$key]['tracking_code_id'] = "";
        $return[$key]['individual'] = ( ( $return[$key]['individual'] == 'yes' ) ? '<span class="text-success" data-feather="check-circle"></span>' : '<span class="text-danger" data-feather="x-circle"></span>') ;
        $return[$key]['ashes'] = ( ( $return[$key]['ashes'] == 'yes' ) ? '<span class="text-success" data-feather="check-circle"></span>' : '<span class="text-danger" data-feather="x-circle"></span>') ;
        /*
        foreach ( $remove_columns as $column )
          unset($list[$key][$column]);
        */
          
        $buttons = ["data" => $data, "action" => ["edit", "delete"],];
        $return[$key]['tracking_code_id'] = '<span class="text-success" data-feather="check-circle"></span>';
        if ( empty($data['tracking_code_id']) ){
          $buttons['action'][] = "add_tracking";
          $return[$key]['tracking_code_id'] = '<span class="text-danger" data-feather="alert-triangle"></span>';
        }
        $return[$key]['buttons'] = $this->create_button($buttons);
        
      }
      return $return;
      return [ "data" => $return ];
    }

    public function add_tracking_code($body_id, $tracking_code_id){
      if ( !$body_id )
        throw new Exception("add_tracking_code Processo não informado, impossível adicionar código de rastreabilidade");

      if ( !$tracking_code_id )
        throw new Exception("Código de rastreablidade não informado, impossível realizar operação");

      $data = $this->list(["id" => $body_id, "limit" => 1]);
      if ( !$data )
        throw new NotFoundException("Cadastro id {$id}  não encontrado, impossível adicionar o código de rastrabilidade");

      $TrackingCode = new TrackingCode();
      $tracking_code = $TrackingCode->list(["id" => $tracking_code_id, "limit" => 1]);
      if ( !$tracking_code )
        throw new NotFoundException("Código de rastreabilidade id {$tracking_code_id}  não encontrado");

      $Crud = new Crud();
      return $Crud->update($table = $this->get_table_name(), ["tracking_code_id" => $tracking_code_id], $where = "ID = :ID", $bind = [ "ID" => $body_id ]);
    }

    public function to_datatable($list){
      $Company = new Company();
      $return = [];
      $columns = ["Rastreabilidade", "Empreendimento", "Nome", "Retorno das Cinzas", "Individual",  "Ações"];
      $items = [];
      //Helper::debug_data($list);
      foreach ( $list as $key => $details ){
        $buttons = ["data" => $details, "action" => ["edit", "delete", "print", "print_auth"]];
        if ( empty($details['tracking_code_id']) )
          $buttons['action'][] = "add_tracking";
        
        $data = json_decode($details['data'], true);
        $company = $Company->list(["id" => $data['company'], "limit" => 1]);
        if ( !$company ){
          $company = [
            ["name" => "", "id" => ""]
          ];
        }

        $temp = [
          [
            "text" => Helper::get_icon(["icon" => ( empty($details['tracking_code_id']) ? "alert" : "ok" )] ), "classes" => ["text-center"],
            "attributes" => ["order" => $details['tracking_code_id'], "raw" => $details['tracking_code_id']],
          ],
          [
            "text" => $company[0]['name'], "classes" => ["text-left"],
            "attributes" => [ "order" => $company[0]['name'], "raw" => $company[0]['name']],
          ],
          [
            "text" => $data['name'], "classes" => ["text-left"],
            "attributes" => [ "order" => $data['name'], "raw" => $data['name']],
          ],
          [
            "text" => Helper::get_icon(["icon" => $data['ashes']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['ashes'], "raw" => $data['ashes']],
          ],
          [
            "text" => Helper::get_icon(["icon" => $data['individual']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['individual'], "raw" => $data['individual']],
          ],
          [
            "text" =>  $this->create_button($buttons), "classes" => ["text-center"],
            "attributes" => ["order" => '', "raw" => '',],
          ]
        ];
        $items[] = $temp;
      }

      return [ "columns" => $columns, "items" => $items ];
    }

  }