<?php

  Class Company extends Base {

    protected const TABLE_NAME = "company";
    protected $id;
    protected $name;
    protected $record;

    public function get_id(){ return $this->id; }
    public function set_id($id): self { $this->id = $id; return $this; }

    public function get_name(){ return $this->name; }
    public function set_name($name): self { $this->name = $name; return $this; }

    public function get_record(){ return $this->record; }
    public function set_record($record): self { $this->record = $record; return $this; }

    public function create(){            
      $Crud = new Crud();
      $new_data = $this->to_array();
      unset($new_data['create_date']);
      unset($new_data['id']);
      return $Crud->create($table = self::TABLE_NAME, $new_data );
    }

    public function edit(){
      $Crud = new Crud(); 
      $new_data = $this->to_array();
      $id = $this->get_id();
      unset($new_data['id']);

      $data = $this->list(["id" => $id, "limit" => 1]);
      
      if ( !$data )
        throw new NotFoundException("Empreendimento id: {$id} não encontrado, impossível editar o registro");

      return $Crud->update($table = self::TABLE_NAME, $new_data, $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    public function delete($id){
      $data = $this->list(["id" => $id, "limit" => 1]);
    
      if ( !$data )
        throw new NotFoundException("Empreendimento id: {$id} não encontrado, impossível excluir o registro");

      $Crud = new Crud();
      return $Crud->delete($table = self::TABLE_NAME, $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    protected function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $class_vars = self::get_class_vars($this);
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND {self::TABLE_NAME}.{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "order":
                $where .= "ORDER BY :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }

    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $field_list = "*";
      if ( !isset($filters['return_all_fields']) || $filters['return_all_fields'] != true ){ 
        if ( !isset($filters['ignore_fields']) || empty($filters['ignore_fields']) ){ // remove password and token
          $ignore_fields = ["password", "api_token"];
        }else {
          $ignore_fields = $filters['ignore_fields'];
        }
        $field_list = $Crud->get_fields(["table" => self::TABLE_NAME, "ignore_fields" => $ignore_fields, "return_query_field_list" => true ] );
      }

      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = self::TABLE_NAME, $where, $bind, $fields = $field_list);
    }

    public static function get_list_options(){
      $Company = new Company();
      $list = $Company->list(["ORDER" => "name asc"]);

      $return = [];
      foreach ( $list as $data ){
        $return[$data['id']] = $data['name'];
      }
      
      return $return;
    }

    public function to_datatable($list){
      $return = [];
      $columns = ["Código", "Nome", "Registro/CNPJ", "Ações"];
      $items = [];
      //Helper::debug_data($list);
      foreach ( $list as $key => $data ){
        $buttons = ["data" => $data, "action" => ["edit", "delete"],];
         
        $temp = [     
          [
            "text" => $data['id'], "classes" => ["text-center"],
            "attributes" => [ "order" => $data['id'], "raw" => $data['id']],
          ],
          [
            "text" => $data['name'], "classes" => ["text-left"],
            "attributes" => [ "order" => $data['name'], "raw" => $data['name'],]
          ],
          [
            "text" => $data['record'], "classes" => ["text-left"],
            "attributes" => [ "order" => $data['record'], "raw" => $data['record'],],
            "format" => "cnpj",
          ],
          /*
          */
          [
            "text" =>  $this->create_button($buttons), "classes" => ["text-center"],
            "attributes" => ["order" => '', "raw" => '',],
          ]
        ];
        $items[] = $temp;
      }

      return [ "columns" => $columns, "items" => $items ];
    }

    public static function get_form($mode = "create"){
      $form = [
          "id" => "form_body",
          "fields" => [
            ['id' => 'id', 'type' => 'hidden', 'required' => true],
            ['id' => 'name',  'label' => 'Razão Social', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 200, "placeholder" => "Informe o Nome"]],  
            ['id' => 'record',  'label' => 'Registro / CNPJ', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 200, "placeholder" => ""]],         
            
          ]
      ];

      switch ( $mode ){
        case "edit":
          $form['fields'][] = [
            "id" => "id", "name" => "id", "type" => "text", "label" => "Id",
            "required" => true ,
            "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
            "attributes" => ["min" => 1 ]
          ];
          break;
      }

      return $form;
    }

    public function create_button($parameters = []){
      
      $buttons = [];
      foreach ( $parameters['action'] as $key => $action ){
        switch ( $action ){
          case  "edit":
            $buttons[] = [ 
              "class" => ["api", "dropdown-item", "btn_edit", "text-dark"], 
              "description" => Helper::get_icon(["icon" => "edit" ]) . " Editar ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                //"data-modal_description" => "Editar Processo: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                "data-bs-toggle" => "modal",
                "data-bs-target" => "#modal_form_company",      
                "data-form_id" => "company",                                
              ],
              "href" => "#",
              "required_permission" => "company_edit",
            ];
            break;

          case  "delete":
            $buttons[] = [ 
              "class" => ["api", "dropdown-item", "btn_delete", "text-danger"], 
              "description" => Helper::get_icon(["icon" => "delete" ]) . " Excluir ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
               // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                //"data-bs-toggle" => "modal",
                //"data-bs-target" => "#modal_form_company",                                      
              ],
              "href" => "#",
              "required_permission" => "company_delete",
            ];
            break;
        }
      }

      if ( !$buttons )
        return "";
      
      return Helper::create_html_button_dropdown($buttons);

    }

  }