<?php

  Class CompanyTrackingCode {

    private $id;
    private $company_id;
    private $tracking_code_id;
  
    private $table_name = "company_tracking_code";

    public function get_id(){ return $this->id; }
    public function set_id($id): self { $this->id = $id; return $this; }

    public function get_company_id(){ return $this->company_id; }
    public function set_company_id($company_id): self { $this->company_id = $company_id; return $this; }

    public function get_tracking_code_id(){ return $this->tracking_code_id; }
    public function set_tracking_code_id($tracking_code_id): self { $this->tracking_code_id = $tracking_code_id; return $this; }

    public function get_table_name(){ return $this->table_name; }

    public function to_object($array){
      $fields = array_keys(get_object_vars($this));
      foreach ( $array as $key => $data ){
        if ( in_array($key, $fields) ){          
          $this->$key = $data;
        }
        
      }

      return $this;
    }

    public function to_array(){            
      return get_object_vars($this);    
    }

    public static function get_class_vars($object){
      return array_keys(get_class_vars(get_class($object))); // $object
    }
    
    public function create(){            
      $Crud = new Crud();
      $new_data = $this->to_array();
      unset($new_data['create_date']);
      unset($new_data['id']);
      return $Crud->create($table = $this->get_table_name(), $new_data );
    }

    public function delete($id){
      $data = $this->list(["id" => $id, "limit" => 1]);
      
      if ( !$data )
        throw new NotFoundException("Vínculo id: {$id} não encontrado, impossível excluir o registro");

      $Crud = new Crud();
      return $Crud->delete($table = $this->get_table_name(), $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    private function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $class_vars = self::get_class_vars($this);
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND {$this->get_table_name()}.{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "order":
                $where .= "ORDER BY :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }

    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $field_list = "*";
      if ( !isset($filters['return_all_fields']) || $filters['return_all_fields'] != true ){ 
        if ( !isset($filters['ignore_fields']) || empty($filters['ignore_fields']) ){ // remove password and token
          $ignore_fields = ["password", "api_token"];
        }else {
          $ignore_fields = $filters['ignore_fields'];
        }
        $field_list = $Crud->get_fields(["table" => $this->get_table_name(), "ignore_fields" => $ignore_fields, "return_query_field_list" => true ] );
      }

      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = $this->get_table_name(), $where, $bind, $fields = $field_list);
    }

    public function list_details($filters = []){
      $Crud = new Crud();
      $sql = " SELECT 
                company_tracking_code.id,
                company_tracking_code.tracking_code_id,
                company.name company_name,
                tracking_code.code,
                company_tracking_code.available,
                bodies.id body_id,
                JSON_VALUE(bodies.data, '$.name') body_name,	
                JSON_VALUE(bodies.data, '$.record') body_record
              
              FROM 
                company_tracking_code 
                
              inner join company on
                company.id = company_tracking_code.company_id 
                
              inner join tracking_code on
                tracking_code.id = company_tracking_code.tracking_code_id 
                
              left join bodies on
                bodies.tracking_code_id = tracking_code.id
                
              WHERE 
                1 = 1

      ";

      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $sql .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->execute_query($sql, $bind);
    }

    public function mark_as_used($id){
      if ( !$id )
        throw new Exception("Id não informado, impossível marcar como utilizado");

      $data = $this->list(["id" => $id, "limit" => 1]);
      if ( !$data )
        throw new NotFoundException("Cadastro id {$id}  não encontrado, impossível marcar como utilizado");

      $Crud = new Crud();
      return $Crud->update($table = $this->get_table_name(), ["available" => "0"], $where = "ID = :ID", $bind = [ "ID" => $id ]);
    }

    public function to_datatable($list){
      $return = [];
      $columns = ["Disponível", "Identificador", "Empreendimento", "Processo", "Ações" ];
      $items = [];
      //Helper::debug_data($list);
      foreach ( $list as $key => $data ){
        $buttons = ["data" => $data, "action" => ["delete", "transfer"],];
        $class_code = ( $data['available'] ? "text-success" : "text-danger" );
        $process = "";
        if ( !empty($data['body_id']) )
          $process = "{$data['body_record']} - {$data['body_name']}";
          
        $temp = [          
          [
            "text" => Helper::get_icon(["icon" => $data['available']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['available'], "raw" => $data['available']],
          ],
          [
            "text" => $data['tracking_code_id'], "classes" => ["text-center", $class_code],
            "attributes" => [ "order" => $data['tracking_code_id'], "raw" => $data['tracking_code_id']],
          ],
          [
            "text" => $data['company_name'], "classes" => ["text-left"],
            "attributes" => [ "order" => $data['company_name'], "raw" => $data['company_name'],]
          ],
          [
            "text" => $process, "classes" => ["text-left"],
            "attributes" => [ "order" => $data['body_name'], "raw" => $data['body_name'], ]
          ],
          /*
          */
          [
            "text" =>  $this->create_button($buttons), "classes" => ["text-center"],
            "attributes" => ["order" => '', "raw" => '',],
          ]
        ];
        $items[] = $temp;
      }

      return [ "columns" => $columns, "items" => $items ];
    }

    public function to_datatable_show($list){
      $return = [];
      $columns = ["Disponível", "Identificador", "Código", "Empreendimento", "Processo", "Ações" ];
      $items = [];
      //Helper::debug_data($list);
      foreach ( $list as $key => $data ){
        $buttons = ["data" => $data, "action" => ["delete", "transfer"],];
        $class_code = ( $data['available'] ? "text-success" : "text-danger" );
        $process = "";
        if ( !empty($data['body_id']) )
          $process = "{$data['body_record']} - {$data['body_name']}";
          
        $temp = [          
          [
            "text" => Helper::get_icon(["icon" => $data['available']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['available'], "raw" => $data['available']],
          ],
          [
            "text" => $data['tracking_code_id'], "classes" => ["text-center", $class_code],
            "attributes" => [ "order" => $data['tracking_code_id'], "raw" => $data['tracking_code_id']],
          ],
          [
            "text" => $data['code'], "classes" => ["text-center", $class_code],
            "attributes" => [ "order" => $data['code'], "raw" => $data['code']],
          ],
          [
            "text" => $data['company_name'], "classes" => ["text-left"],
            "attributes" => [ "order" => $data['company_name'], "raw" => $data['company_name'],]
          ],
          [
            "text" => $process, "classes" => ["text-left"],
            "attributes" => [ "order" => $data['body_name'], "raw" => $data['body_name'], ]
          ],
          /*
          */
          [
            "text" =>  $this->create_button($buttons), "classes" => ["text-center"],
            "attributes" => ["order" => '', "raw" => '',],
          ]
        ];
        $items[] = $temp;
      }

      return [ "columns" => $columns, "items" => $items ];
    }

    public function create_button($parameters = []){
      
      $buttons = [];
      foreach ( $parameters['action'] as $key => $action ){
        switch ( $action ){
          case  "edit":
            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_edit", "text-dark"), 
              "description" => Helper::get_icon(["icon" => "edit" ]) . " Editar ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                //"data-modal_description" => "Editar Processo: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                "data-bs-toggle" => "modal",
                "data-bs-target" => "#modal_form_company_tracking_code",      
                "data-form_id" => "company_tracking_code",                                
              ],
              "href" => "#",
              "required_permission" => "company_edit",
            ];
            break;

          case  "delete":
            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_delete", "text-danger"), 
              "description" => Helper::get_icon(["icon" => "delete" ]) . " Excluir ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
               // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                //"data-bs-toggle" => "modal",
                //"data-bs-target" => "#modal_form_company_tracking_code",                                      
              ],
              "href" => "#",
              "required_permission" => "company_delete",
            ];
            break;

          case  "transfer":
            if ( !$parameters['data']['available'] )
              break;

            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_transfer", "text-info"), 
              "description" => Helper::get_icon(["icon" => "repeat" ]) . " Transferência entre Empreendimentos ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                "data-bs-toggle" => "modal",
                "data-bs-target" => "#modal_form_company_tracking_code_transfer",         
                "data-form_id" => "company_tracking_code_transfer",                             
              ],
              "href" => "#",
              "required_permission" => "company_delete",
            ];
            break;
        }
      }

      if ( !$buttons )
        return "";
      
      return Helper::create_html_button_dropdown($buttons);

    }

    public static function get_form($mode = "create"){
      $form = [
          "id" => "form_company_tracking_code",
          "fields" => [
            ['id' => 'id', 'type' => 'hidden', 'required' => true],  
              
            ['id' => 'company_id','label' => 'Empreendimento', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...',
              ], 
              'options' => Company::get_list_options()], 

            ['id' => 'tracking_code_id',  'label' => 'Identificador do Envelope', 'required' => true,
              'type' => 'text', 'attributes' => ['minlength' => 1, 'maxlength' => 200, "placeholder" => "Informe o código presente no verso do envelope"]], 
            
          ]
      ];
      return $form;
    }

    public static function get_form_transfer($mode = "create"){
      $form = [
          "id" => "form_company_tracking_code_transfer",
          "fields" => [
            ['id' => 'id', 'type' => 'hidden', 'required' => true],  
              
            ['id' => 'tracking_code_id',  'label' => 'Identificador do Envelope', 'required' => true,
            'type' => 'text', 'attributes' => ["readonly" => true, 'minlength' => 1, 'maxlength' => 200, "placeholder" => "Informe o código presente no verso do envelope"]], 
          
            ['id' => 'company_id_from','label' => 'Empreendimento Atual', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...',
              "readonly" => true, "disabled" => true], 
              'options' => Company::get_list_options()], 
              
            ['id' => 'company_id_to','label' => 'Novo Empreendimento', 'required' => true,
              'type' => 'select', 'attributes' => ['emptyval' => 'Selecione...',
              ], 
              'options' => Company::get_list_options()], 

          ]
      ];
      return $form;
    }


    public function transfer($parameters){
      if ( !$parameters['id'] )
        throw new Exception("Id não informado, impossível transferir");

      if ( !$parameters['company_id_from'] )
        throw new Exception("Empreendimento de origem não informado, impossível transferir");

      if ( !$parameters['company_id_to'] )
        throw new Exception("Empreendimento de destino não informado, impossível transferir");

      $data = $this->list(["id" => $parameters['id'], "company_id" => $parameters['company_id_from'], "limit" => 1]);
      if ( !$data )
        throw new NotFoundException("Cadastro id {$id}  não encontrado, impossível transferir");
      
      $Crud = new Crud();
      return $Crud->update($table = $this->get_table_name(), ["company_id" => $parameters['company_id_to']], $where = "ID = :ID", $bind = [ "ID" => $parameters['id'] ]);
    }
  }