<?php

  Class Permission {

    private $id;
    private $code;
    private $description;

    private $table_name = "permission";
    public function get_id(){ return $this->id; }
    public function set_id($id): self { $this->id = $id; return $this; }

    public function get_code(){ return $this->code; }
    public function set_code($code): self { $this->code = $code; return $this; }

    public function get_description(){ return $this->description; }
    public function set_description($description): self { $this->description = $description; return $this; }

    public function get_table_name(){ return $this->table_name; }

    public function to_object($array){
      $fields = array_keys(get_object_vars($this));
      foreach ( $array as $key => $data ){
        if ( in_array($key, $fields) ){
          $this->$key = $data;
        }
        
      }

      return $this;
    }

    public function to_array(){            
      return get_object_vars($this);    
    }

    public static function get_class_vars($object){
      return array_keys(get_class_vars(get_class($object))); // $object
    }

    public function create(){            
      $Crud = new Crud();
      $data = $this->to_array();

      if ( empty($data['code']) )
        throw new Exception("Código não informado, impossível criar o cadastro");

      if ( empty($data['description']) )
        throw new Exception("Código não informado, impossível criar o cadastro");
      
      return $Crud->create($table = $this->get_table_name(), $data );
    }

    public function edit(){
      $Crud = new Crud();
      $new_data = $this->to_array();
      $id = $new_data['id'];
      unset($new_data['id']);
      return $Crud->update($table = $this->get_table_name(), $new_data, $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    public function delete($id){
      $data = $this->list(["id" => $id, "limit" => 1]);
      
      if ( !$data )
        throw new NotFoundException("Permissão id: {$id} não encontrada, impossível excluir o registro");
        
      $Crud = new Crud();
      return $Crud->delete($table = $this->get_table_name(), $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    private function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $class_vars = self::get_class_vars($this);
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND {$this->get_table_name()}.{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "order":
                $where .= "ORDER BY :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }

    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = $this->get_table_name(), $where, $bind, $fields="*");
    }

    public static function get_form($mode = "create"){

      switch ( $mode ){
        case "edit":
        case "create";
        default:
          $form = [
            "id" => "form_permission",
            "fields" => [      
              [
                "id" => "id", "name" => "id", "type" => "text", "label" => "Id",
                "required" => true ,
                "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
                "attributes" => ["min" => 1 ]
              ],
              [
                "id" => "code", "name" => "code", "type" => "text", "label" => "Código",
                "required" => true,
                "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
                "attributes" => ["min" => 1 ]
              ],
              [
                "id" => "description", "name" => "description", "type" => "text", "label" => "Descrição",
                "required" => true,
                "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
                "attributes" => ["min" => 1 ]
              ],
              [
                "id" => "active", "name" => "active", "type" => "select", "label" => "Status",
                "required" => true,
                "classes" => [ "form-control", "input-sm", "form-select", "limpar", "form-control-sm", ],
                "attributes" => [ "data-db-origin" => "active", "emptyval" => "Selecione" ],
                "options" => [ 
                  "1" => "Ativo",
                  "0" => "Bloqueado",
                ]
              ]
            ]
        ];

        break;
      }
      return $form;
    }
  }