<?php

  Class TrackingCode {

    private $id;
    private $code;
    private $used;
    private $scrapped;
    private $shipped;
    private $printed;

    private $table_name = "tracking_code";

    public function get_id(){ return $this->id; }
    public function set_id($id): self { $this->id = $id; return $this; }

    public function get_code(){ return $this->code; }
    public function set_code($code): self { $this->code = $code; return $this; }

    public function get_used(){ return $this->used; }
    public function set_used($used): self { $this->used = $used; return $this; }

    public function get_scrapped(){ return $this->scrapped; }
    public function set_scrapped($scrapped): self { $this->scrapped = $scrapped; return $this; }

    public function get_shipped(){ return $this->shipped; }
    public function set_shipped($shipped): self { $this->shipped = $shipped; return $this; }

    public function get_printed(){ return $this->printed; }
    public function set_printed($printed): self { $this->printed = $printed; return $this; }

    public function get_table_name(){ return $this->table_name; }

    public function to_object($array){
      $fields = array_keys(get_object_vars($this));
      foreach ( $array as $key => $data ){
        if ( in_array($key, $fields) ){          
          $this->$key = $data;
        }
        
      }

      return $this;
    }

    public function to_array(){            
      return get_object_vars($this);    
    }

    public static function get_class_vars($object){
      return array_keys(get_class_vars(get_class($object))); // $object
    }

    private function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $class_vars = self::get_class_vars($this);
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND {$this->get_table_name()}.{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "id_from":
                $where .= "\r\n AND {$this->get_table_name()}.id >= :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "id_to":
                $where .= "\r\n AND {$this->get_table_name()}.id <= :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "order":
                $where .= "ORDER BY :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }

    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $field_list = "*";
      if ( !isset($filters['return_all_fields']) || $filters['return_all_fields'] != true ){ 
        if ( !isset($filters['ignore_fields']) || empty($filters['ignore_fields']) ){ // remove password and token
          $ignore_fields = ["password", "api_token"];
        }else {
          $ignore_fields = $filters['ignore_fields'];
        }
        $field_list = $Crud->get_fields(["table" => $this->get_table_name(), "ignore_fields" => $ignore_fields, "return_query_field_list" => true ] );
      }

      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = $this->get_table_name(), $where, $bind, $fields = $field_list);
    }

    public function mark_as($id, $mark = false){
      if ( !$id )
        throw new Exception("Id não informado, impossível atualizar o cadastro ");

      if ( !$mark )
        throw new Exception("Tipo de marcação não informada, impossível atualizar o cadastro");

      $data = $this->list(["id" => $id, "limit" => 1]);
      if ( !$data )
        throw new NotFoundException("Código de Rastreamento id {$id}  não encontrado, impossível realizar operação");

      $mark_list = [ "printed", "shipped", "scrapped", "used" ];
      if ( !in_array($mark, $mark_list) )
        throw new NotFoundException("Tipo de marcação {$mark} inválido");

      $update_fields = [$mark => "1"];

      $Crud = new Crud();
      return $Crud->update($table = $this->get_table_name(), $update_fields, $where = "ID = :ID", $bind = [ "ID" => $id ]);
    }

    public function to_datatable($list){
      $return = [];
      $columns = ["Identificador", "Código", "Impresso", "Enviado", "Descartado", "Usado",  "Ações" ];
      $items = [];
      //Helper::debug_data($list);
      foreach ( $list as $key => $data ){
        $buttons = ["data" => $data, "action" => ["mark_used", "mark_shipped", "mark_scrapped", "mark_printed"],];
        $class_code = "";
          
        $temp = [ 
          
          [
            "text" => $data['id'], "classes" => ["text-center", $class_code],
            "attributes" => [ "order" => $data['id'], "raw" => $data['id']],
          ],     
          [
            "text" => $data['code'], "classes" => ["text-center", $class_code],
            "attributes" => [ "order" => $data['code'], "raw" => $data['code']],
          ],         
          [
            "text" => Helper::get_icon(["icon" => $data['printed']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['printed'], "raw" => $data['printed']],
          ],     
          [
            "text" => Helper::get_icon(["icon" => $data['shipped']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['shipped'], "raw" => $data['shipped']],
          ],     
          [
            "text" => Helper::get_icon(["icon" => $data['scrapped']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['scrapped'], "raw" => $data['scrapped']],
          ],     
          [
            "text" => Helper::get_icon(["icon" => $data['used']]), "classes" => ["text-center"],
            "attributes" => [  "order" => $data['used'], "raw" => $data['used']],
          ],
          [
            "text" =>  $this->create_button($buttons), "classes" => ["text-center"],
            "attributes" => ["order" => '', "raw" => '',],
          ]
        ];
        $items[] = $temp;
      }

      return [ "columns" => $columns, "items" => $items ];
    }

    public function create_button($parameters = []){
      
      $buttons = [];
      foreach ( $parameters['action'] as $key => $action ){
        switch ( $action ){
          case  "edit":
            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_edit", "text-dark"), 
              "description" => Helper::get_icon(["icon" => "edit" ]) . " Editar ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                //"data-modal_description" => "Editar Processo: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                "data-bs-toggle" => "modal",
                "data-bs-target" => "#modal_form_company_tracking_code",      
                "data-form_id" => "company_tracking_code",                                
              ],
              "href" => "#",
              "required_permission" => "company_edit",
            ];
            break;

          case  "delete":
            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_delete", "text-danger"), 
              "description" => Helper::get_icon(["icon" => "delete" ]) . " Excluir ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
               // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                //"data-bs-toggle" => "modal",
                //"data-bs-target" => "#modal_form_company_tracking_code",                                      
              ],
              "href" => "#",
              "required_permission" => "company_delete",
            ];
            break;

          case  "transfer":
            if ( !$parameters['data']['available'] )
              break;

            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_transfer", "text-info"), 
              "description" => Helper::get_icon(["icon" => "repeat" ]) . " Transferência entre Empreendimentos ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                // "data-modal_description" => "Excluir Usuário: {$parameters['data']['id']} - {$parameters['data']['name']}",                
                "data-bs-toggle" => "modal",
                "data-bs-target" => "#modal_form_company_tracking_code_transfer",         
                "data-form_id" => "company_tracking_code_transfer",                             
              ],
              "href" => "#",
              "required_permission" => "company_delete",
            ];
            break;

          case  "mark_printed":
            if ( $parameters['data']['printed'] )
              break;

            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_mark", ), 
              "description" => Helper::get_icon(["icon" => "printer" ]) . "Marcar como Impresso ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                "data-mark" => "printed",                 
              ],
              "href" => "#",
              //"required_permission" => "tracking_code_mark",
            ];
            break;

          case  "mark_shipped":
            if ( $parameters['data']['shipped'] )
              break;

            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_mark", ), 
              "description" => Helper::get_icon(["icon" => "repeat" ]) . "Marcar como Enviado ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                "data-mark" => "shipped",                 
              ],
              "href" => "#",
              //"required_permission" => "tracking_code_mark",
            ];
            break;

          case  "mark_scrapped":
            if ( $parameters['data']['mark_scrapped'] )
              break;

            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_mark", "text-danger"), 
              "description" => Helper::get_icon(["icon" => "trash" ]) . "Marcar como Descartado ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                "data-mark" => "scrapped",                 
              ],
              "href" => "#",
              //"required_permission" => "tracking_code_mark",
            ];
            break;

          case  "mark_used":
            if ( $parameters['data']['used'] )
              break;

            $buttons[] = [ 
              "class" => array("api", "dropdown-item", "btn_mark", "text-info"), 
              "description" => Helper::get_icon(["icon" => "repeat" ]) . "Marcar como Utilizado ", 
              "type" => "link",
              "attributes" => [
                "data-object" => get_class($this),
                "data-action" => $action,
                "data-id" => $parameters['data']['id'],
                "data-object_info" => json_encode($parameters['data'], true),
                "data-mark" => "used",                 
              ],
              "href" => "#",
              //"required_permission" => "tracking_code_mark",
            ];
            break;
        }
      }

      if ( !$buttons )
        return "";
      
      return Helper::create_html_button_dropdown($buttons);

    }
  }