<?php

  Class UserPermission {

    private $id;
    private $user_id;
    private $permission_id;

    private $table_name = "user_permission";
    public function get_id(){ return $this->id; }
    public function set_id($id): self { $this->id = $id; return $this; }

    public function get_permission_id(){ return $this->permission_id; }
    public function set_permission_id($permission_id): self { $this->permission_id = $permission_id; return $this; }

    public function get_table_name(){ return $this->table_name; }

    public function to_object($array){
      $fields = array_keys(get_object_vars($this));
      foreach ( $array as $key => $data ){
        if ( in_array($key, $fields) ){
          $this->$key = $data;
        }
        
      }

      return $this;
    }

    public function to_array(){            
      return get_object_vars($this);    
    }

    public static function get_class_vars($object){
      return array_keys(get_class_vars(get_class($object))); // $object
    }

    public function create(){            
      $Crud = new Crud();
      $data = $this->to_array();

      if ( empty($data['user_id']) )
        throw new Exception("Usuário não informado, impossível criar o cadastro");

      if ( empty($data['permission_id']) )
        throw new Exception("Permissão não informada, impossível criar o cadastro");
      
      return $Crud->create($table = $this->get_table_name(), $data );
    }

    public function edit(){
      $Crud = new Crud();
      $new_data = $this->to_array();
      $id = $new_data['id'];
      unset($new_data['id']);
      return $Crud->update($table = $this->get_table_name(), $new_data, $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    public function delete($id){
      $data = $this->list(["id" => $id, "limit" => 1]);
      
      if ( !$data )
        throw new NotFoundException("Permissão id: {$id} não encontrada, impossível excluir o registro");
        
      $Crud = new Crud();
      return $Crud->delete($table = $this->get_table_name(), $where = "id = :id", $bind = [ ":id" => $id ]);
    }

    private function create_sql_filter($filters = []){
      $where = "";
      $bind = [];
      $class_vars = self::get_class_vars($this);
      if ( $filters ){
        foreach ( $filters as $key => $filter ){
          if ( in_array($key, $class_vars) ){ // create where based on class attributes 
            $where .= "\r\n AND {$this->get_table_name()}.{$key} = :{$key} ";
            $bind[":{$key}"] = $filter;
          }else {
            switch ( $key ){
              case "order":
                $where .= "ORDER BY :{$key}";
                $bind[":{$key}"] = $filter;
                break;

              case "limit":
                $where .= "\r\n LIMIT :{$key}";
                $bind[":{$key}"] = $filter;
                break;
            }
          }
        }
      }

      return [ "where" => $where, "bind" => $bind ];
    }
    public function list($filters = []){
      $Crud = new Crud();

      $where = ( !empty($filters) ?  " \r\n 1 = 1 " : "" );
      $bind = [];
      if ( $filters ){
        $sql_filter = self::create_sql_filter($filters);
        $where .= $sql_filter["where"];
        $bind = $sql_filter["bind"];
      }
      return $Crud->read($table = $this->get_table_name(), $where, $bind, $fields="*");
    }

    public function get_form(){
      return [
          "id" => "form_permission",
          "fields" => [      
            [
              "id" => "id", "name" => "id", "type" => "text", "label" => "Id",
              "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
              "attributes" => ["min" => 1 ]
            ],      
            [
              "id" => "code", "name" => "code", "type" => "text", "label" => "Código",
              "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
              "attributes" => ["min" => 1 ]
            ],
            [
              "id" => "description", "name" => "description", "type" => "text", "label" => "Descrição",
              "classes" => [ "form-control", "input-sm", "text-left", "limpar", "form-control-sm", ],
              "attributes" => ["min" => 1 ]
            ],
          ]
      ];
    }

  }